@@ -324,7 +324,7 @@ static LPWSTR service_get_pipe_name(void)
     RegCloseKey(service_current_key);
     if (ret != ERROR_SUCCESS || type != REG_DWORD)
         return NULL;
-    len = sizeof(format)/sizeof(WCHAR) + 10 /* strlenW("4294967295") */;
+    len = ARRAY_SIZE(format) + 10 /* strlenW("4294967295") */;
     name = heap_alloc(len * sizeof(WCHAR));
     if (!name)
         return NULL;
@@ -1688,6 +1688,7 @@ BOOL WINAPI QueryServiceConfig2W(SC_HANDLE hService, DWORD dwLevel, LPBYTE buffe
 
     if (!needed)
     {
+        if (dwLevel == SERVICE_CONFIG_DESCRIPTION) heap_free(bufptr);
         SetLastError(ERROR_INVALID_ADDRESS);
         return FALSE;
     }
