@@ -72,8 +72,8 @@ char *WtoA(LPCWSTR wstr)
 
 static BOOL get_install_root(LPWSTR install_dir)
 {
-    const WCHAR dotnet_key[] = {'S','O','F','T','W','A','R','E','\\','M','i','c','r','o','s','o','f','t','\\','.','N','E','T','F','r','a','m','e','w','o','r','k','\\',0};
-    const WCHAR install_root[] = {'I','n','s','t','a','l','l','R','o','o','t',0};
+    static const WCHAR dotnet_key[] = {'S','O','F','T','W','A','R','E','\\','M','i','c','r','o','s','o','f','t','\\','.','N','E','T','F','r','a','m','e','w','o','r','k','\\',0};
+    static const WCHAR install_root[] = {'I','n','s','t','a','l','l','R','o','o','t',0};
 
     DWORD len;
     HKEY key;
@@ -564,7 +564,7 @@ BOOL WINAPI StrongNameSignatureVerificationEx(LPCWSTR filename, BOOL forceVerifi
 
 HRESULT WINAPI CreateDebuggingInterfaceFromVersion(int nDebugVersion, LPCWSTR version, IUnknown **ppv)
 {
-    const WCHAR v2_0[] = {'v','2','.','0','.','5','0','7','2','7',0};
+    static const WCHAR v2_0[] = {'v','2','.','0','.','5','0','7','2','7',0};
     HRESULT hr = E_FAIL;
     ICLRRuntimeInfo *runtimeinfo;
 
@@ -682,7 +682,7 @@ static BOOL install_wine_mono(void)
     LONG len;
     BOOL ret;
 
-    static const char* mono_version = "4.7.3";
+    static const char* mono_version = "4.7.5";
     static const char* mono_upgrade_code = "{DE624609-C6B5-486A-9274-EF0B854F6BC5}";
 
     static const WCHAR controlW[] = {'\\','c','o','n','t','r','o','l','.','e','x','e',0};
