@@ -288,9 +288,9 @@ static HANDLE engine_thread;
 
 DWORD WINAPI engine_threadproc(void *user)
 {
+    static const SLEngineOption options[] = { {SL_ENGINEOPTION_THREADSAFE, SL_BOOLEAN_TRUE} };
     HANDLE evt = user;
     SLresult sr;
-    SLEngineOption options[] = { {SL_ENGINEOPTION_THREADSAFE, SL_BOOLEAN_TRUE} };
 
     sr = pslCreateEngine(&sl, 1, options, 0, NULL, NULL);
     if(sr != SL_RESULT_SUCCESS){
