@@ -128,7 +128,7 @@ static struct gl_drawable *create_gl_drawable( HWND hwnd, HDC hdc, int format )
     gl->hwnd   = hwnd;
     gl->hdc    = hdc;
     gl->format = format;
-    gl->window = create_ioctl_window( hwnd, TRUE );
+    gl->window = create_ioctl_window( hwnd, TRUE, 1.0f );
     gl->surface = 0;
     gl->pbuffer = p_eglCreatePbufferSurface( display, pixel_formats[gl->format - 1].config, attribs );
     EnterCriticalSection( &drawable_section );
@@ -389,13 +389,6 @@ static BOOL android_wglSwapIntervalEXT( int interval )
         SetLastError(ERROR_INVALID_DATA);
         return FALSE;
     }
-    TRACE("p_eglSwapInterval %p\n", p_eglSwapInterval);
-    if (!p_eglSwapInterval || 1)
-    {
-        ERR("eglSwapInterval is NULL!\n");
-        SetLastError( ERROR_INVALID_FUNCTION );
-        return FALSE;
-    }
 
     ret = p_eglSwapInterval( display, interval );
 
