@@ -105,6 +105,21 @@ static inline int get_dib_image_size( const BITMAPINFO *info )
 }
 
 
+/**********************************************************************
+ *	     get_win_monitor_dpi
+ */
+static UINT get_win_monitor_dpi( HWND hwnd )
+{
+    DPI_AWARENESS_CONTEXT context;
+    UINT ret;
+
+    context = SetThreadDpiAwarenessContext( DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE );
+    ret = GetDpiForSystem();  /* FIXME: get monitor dpi */
+    SetThreadDpiAwarenessContext( context );
+    return ret;
+}
+
+
 /***********************************************************************
  *           alloc_win_data
  */
@@ -115,7 +130,8 @@ static struct android_win_data *alloc_win_data( HWND hwnd )
     if ((data = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*data))))
     {
         data->hwnd = hwnd;
-        data->window = create_ioctl_window( hwnd, FALSE );
+        data->window = create_ioctl_window( hwnd, FALSE,
+                                            (float)get_win_monitor_dpi( hwnd ) / GetDpiForWindow( hwnd ));
         EnterCriticalSection( &win_data_section );
         win_data_context[context_idx(hwnd)] = data;
     }
@@ -467,6 +483,7 @@ static void pull_events(void)
  */
 static int process_events( DWORD mask )
 {
+    DPI_AWARENESS_CONTEXT context;
     struct java_event *event, *next, *previous;
     unsigned int count = 0;
 
@@ -507,11 +524,13 @@ static int process_events( DWORD mask )
         {
         case DESKTOP_CHANGED:
             TRACE( "DESKTOP_CHANGED %ux%u\n", event->data.desktop.width, event->data.desktop.height );
+            context = SetThreadDpiAwarenessContext( DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE );
             screen_width = event->data.desktop.width;
             screen_height = event->data.desktop.height;
             init_monitors( screen_width, screen_height );
             SetWindowPos( GetDesktopWindow(), 0, 0, 0, screen_width, screen_height,
                           SWP_NOZORDER | SWP_NOACTIVATE | SWP_NOREDRAW );
+            SetThreadDpiAwarenessContext( context );
             break;
 
         case CONFIG_CHANGED:
@@ -548,7 +567,6 @@ static int process_events( DWORD mask )
                     RECT rect;
                     SetRect( &rect, event->data.motion.input.u.mi.dx, event->data.motion.input.u.mi.dy,
                              event->data.motion.input.u.mi.dx + 1, event->data.motion.input.u.mi.dy + 1 );
-                    MapWindowPoints( 0, event->data.motion.hwnd, (POINT *)&rect, 2 );
 
                     SERVER_START_REQ( update_window_zorder )
                     {
@@ -738,14 +756,6 @@ static void set_color_info( BITMAPINFO *info, BOOL has_alpha )
     colors[2] = 0x0000ff;
 }
 
-static BOOL surface_has_alpha( struct android_window_surface *surface )
-{
-    return (surface->region_data ||
-            surface->info.bmiHeader.biCompression == BI_RGB ||
-            surface->alpha != 255 ||
-            surface->color_key != CLR_INVALID);
-}
-
 /* apply the window region to a single line of the destination image. */
 static void apply_line_region( DWORD *dst, int width, int x, int y, const RECT *rect, const RECT *end )
 {
@@ -1005,7 +1015,6 @@ done:
     HeapFree( GetProcessHeap(), 0, surface->region_data );
     surface->region_data = data;
     *window_surface->funcs->get_bounds( window_surface ) = surface->header.rect;
-    ioctl_set_window_alpha( surface->hwnd, surface_has_alpha( surface ));
     window_surface->funcs->unlock( window_surface );
     if (region != win_region) DeleteObject( region );
 }
@@ -1047,7 +1056,6 @@ static struct window_surface *create_surface( HWND hwnd, const RECT *rect,
     TRACE( "created %p hwnd %p %s bits %p-%p\n", surface, hwnd, wine_dbgstr_rect(rect),
            surface->bits, (char *)surface->bits + surface->info.bmiHeader.biSizeImage );
 
-    ioctl_set_window_alpha( hwnd, surface_has_alpha( surface ));
     return &surface->header;
 
 failed:
@@ -1072,15 +1080,12 @@ static void set_surface_layered( struct window_surface *window_surface, BYTE alp
     surface->alpha = alpha;
     set_color_key( surface, color_key );
     if (alpha != prev_alpha || surface->color_key != prev_key)  /* refresh */
-    {
         *window_surface->funcs->get_bounds( window_surface ) = surface->header.rect;
-        ioctl_set_window_alpha( surface->hwnd, surface_has_alpha( surface ));
-    }
     window_surface->funcs->unlock( window_surface );
 }
 
 /***********************************************************************
- *              get_mono_icon_argb
+ *           get_mono_icon_argb
  *
  * Return a monochrome icon/cursor bitmap bits in ARGB format.
  */
@@ -1111,7 +1116,7 @@ static unsigned int *get_mono_icon_argb( HDC hdc, HBITMAP bmp, unsigned int *wid
             else if (xor && !and)
                 *ptr = 0xffffffff;
             else
-                /* We can't draw "invert" pixels, so render them as black instead. */
+                /* we can't draw "invert" pixels, so render them as black instead */
                 *ptr = 0xff000000;
         }
 
@@ -1123,11 +1128,10 @@ done:
     return bits;
 }
 
-
 /***********************************************************************
- *              get_bitmap_argb
+ *           get_bitmap_argb
  *
- * Return the bitmap bits in ARGB format. Helper for setting icon hints.
+ * Return the bitmap bits in ARGB format. Helper for setting icons and cursors.
  */
 static unsigned int *get_bitmap_argb( HDC hdc, HBITMAP color, HBITMAP mask, unsigned int *width,
                                       unsigned int *height )
@@ -1579,7 +1583,7 @@ void CDECL ANDROID_SetParent( HWND hwnd, HWND parent, HWND old_parent )
     TRACE( "win %p parent %p -> %p\n", hwnd, old_parent, parent );
 
     data->parent = (parent == GetDesktopWindow()) ? 0 : parent;
-    ioctl_set_window_parent( hwnd, parent );
+    ioctl_set_window_parent( hwnd, parent, (float)get_win_monitor_dpi( hwnd ) / GetDpiForWindow( hwnd ));
     release_win_data( data );
 }
 
